library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity artix_ftdi_fifo0 is port(
	CLK: in std_logic; --100MHz 
	D: inout std_logic_vector(7 downto 0);
	RXF: in std_logic;
	TXE: in std_logic;
	RD: out std_logic;
	WR: out std_logic;
	SIWU: out std_logic;
	CLKOUT: in std_logic; --60MHz FTDI
	OE: out std_logic;
	PWRSAV: out std_logic;
	sysled: out std_logic);
end artix_ftdi_fifo0;

architecture behav of artix_ftdi_fifo0 is

component fifo_generator_0 IS PORT(
    wr_clk : IN STD_LOGIC;
    rd_clk : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    wr_en : IN STD_LOGIC;
    rd_en : IN STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    full : OUT STD_LOGIC;
    empty : OUT STD_LOGIC);
END component;

signal blinkcounter_60: unsigned(25 downto 0):=(others => '0');
signal WR_buff, TXE_buff, fail: std_logic:='0';
signal fifo_full, fifo_empty, fifo_empty_buff, fifo_rd, fifo_rd_buff, fifo_wr: std_logic:='0';
signal send_byte: unsigned(7 downto 0):=(others => '0');
signal fifo_din, fifo_dout: std_logic_vector(7 downto 0):=(others => '0');
signal gen_counter: integer range 0 to 99:=0;

begin

fifo0: fifo_generator_0 PORT MAP(
    wr_clk => CLK, --100MHz
    rd_clk => CLKOUT,--60MHz
    din => fifo_din,
    wr_en => fifo_wr,
    rd_en => fifo_rd,
    dout => D,
    full => fifo_full,
    empty => fifo_empty);
	
RD <= '1';
SIWU <= '1';
OE <= '1';
PWRSAV <= 'Z';
sysled <= blinkcounter_60(25);

process 
	begin   
	wait until rising_edge(CLK);
	gen_counter <= gen_counter +1;
	fifo_wr <= '0';
	if gen_counter = 99 then
		gen_counter <= 0;
	end if;
	if gen_counter < 35 then
		fifo_wr <= '1';
		send_byte <= send_byte +1;
		fifo_din <= std_logic_vector(send_byte);
	end if;
end process;

WR <= '0' when (TXE = '0' and fifo_rd_buff = '1') or (TXE = '0' and fail = '1') else '1';
fifo_rd <= '1' when TXE = '0' and fifo_empty = '0' else '0';
process 
	begin   
	wait until rising_edge(CLKOUT);--CLK60);
	fifo_rd_buff <= fifo_rd;
	TXE_buff <= TXE;
	fifo_empty_buff <= fifo_empty;
	if TXE = '0' then
		fail <= '0';
	end if;
	if fifo_empty_buff = '0' and TXE = '1' and TXE_buff = '0' then
		fail <= '1';
	end if;

	blinkcounter_60 <= blinkcounter_60 +1;
end process;

end behav;